@@ -225,13 +225,18 @@ class EventHub(object):
             # Set flag to indicate disconnection was intentional.
             self._intentional_disconnect = True
 
-            # # Unsubscribe all subscribers.
+            # Set blocking to true on socket to make sure unsubscribe events
+            # are emitted before closing the connection.
+            self._connection.sock.setblocking(1)
+
+            # Unsubscribe all subscribers.
             if unsubscribe:
                 for subscriber in self._subscribers[:]:
                     self.unsubscribe(subscriber.metadata['id'])
 
-                # Wait briefly to allow unsubscribe messages to be sent.
-                time.sleep(self._wait_timeout)
+            # Now disconnect.
+            self._connection.close()
+            self._connection = None
 
             # Shutdown background processing thread.
             self._processor_thread.cancel()
@@ -241,10 +246,6 @@ class EventHub(object):
             if threading.current_thread() != self._processor_thread:
                 self._processor_thread.join(self._wait_timeout)
 
-            # Now disconnect.
-            self._connection.close()
-            self._connection = None
-
     def reconnect(self, attempts=10, delay=5):
         '''Reconnect to server.
 
